/* Class PersonalOptions */

/*
    NUT nutrition software 
    Copyright (C) 1996-2012 by Jim Jozwiak.

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

#include "Nut.h"
#include "PersonalOptions.h"
#include "DVValueOutput.h"
#include "db.h"
#include "options.h"
#include "recmeal.h"
#include "Nut_Box.h"
#include <FL/Fl.H>
#include <stdint.h>

struct food *foodwork;
DVValueOutput *cvo, *ovo, *lvo, *mvo, *svo, *fvo, *nvo, *pvo;

static void lock_cb(Fl_Widget *w, void *data)
{
Nut_LightButton *lb = (Nut_LightButton *) w;
intptr_t nut = (intptr_t) data; 
double value = lb->value();
options.locknuts[nut] = (int) value;
if (nut == ENERC_KCAL && options.autocal != 2) options.autocal = 0;
if (nut == ENERC_KCAL && options.autocal == 2) lb->value(0);
if (nut == PROCNT) options.pcprotein = 0;
if (nut == CHOCDF) options.pccarb = 0;
write_OPTIONS();
}

static void alock_cb(Fl_Widget *w, void *data)
{
Nut_LightButton *lb = (Nut_LightButton *) w;
Nut_LightButton *la = (Nut_LightButton *) data;
double value = lb->value();
if (value > 0)
 {
 options.autocal = 2;
 options.wltweak = 0;
 options.locknuts[ENERC_KCAL] = 1;
 la->value(0);
 }
else
 {
 options.autocal = 0;
 options.wltweak = 0;
 options.locknuts[ENERC_KCAL] = 0;
 }
write_OPTIONS();
}

static void weightlog_cb(Fl_Widget *w, void *who)
{
PersonalOptions *who_we_are = (PersonalOptions *) who;
who_we_are->initializeWeightLog();
}

static void settings_cb(Fl_Widget *w, void *who)
{
PersonalOptions *who_we_are = (PersonalOptions *) who;
who_we_are->prev();
}

static void restore_global_defaults_cb(Fl_Widget *whatever, void *who)
{
restore_defaults(1, foodwork);
PersonalOptions *us_now = (PersonalOptions *) who;
us_now->update();
dv_change();
}

PersonalOptions::PersonalOptions (int x, int y, int w, int h, Fl_Color widgetcolor) : Fl_Wizard (x, y, w, h)
{
int column[4] = {31*w/128, 31*w/128+w/50+w/6, 31*w/128+5*w/50+w/6+w/75+w/15, 31*w/128+w/50+w/6+w/75+w/5+3*w/50};
int row[9];
this->label("Options");
this->labeltype(FL_ENGRAVED_LABEL);
for (int i = 0; i < 9; i++) row[i] = 4*h/22+i*2*h/22;
Fl_Group *settings = new Fl_Group(x, y, w, h);
settings->color(widgetcolor);
 {
 Nut_Button *o = new Nut_Button(x+96*w/128, y+h/640+h/22, 9*w/40, h/22, "Weight Log Regression");
 o->color(fl_lighter(widgetcolor));
 o->labelfont(FL_BOLD);
 o->labelcolor(fl_darker(FL_RED));
 o->clear_visible_focus();
 o->box(FL_ROUND_UP_BOX);
 o->callback(weightlog_cb,this);
 }
 {
 Nut_Box *o = new Nut_Box(column[0], row[0], w/6, h/22, "Calories");
 o->labelcolor(FL_YELLOW);
 o->labelfont(FL_BOLD);
 o->align(FL_ALIGN_INSIDE|FL_ALIGN_LEFT);
 }
cvo = new DVValueOutput(column[1], row[0], w/10, h/22, widgetcolor, ENERC_KCAL);
clock = new Nut_LightButton(column[2], row[0], w/15, h/22, "LOCK");
clock->color(widgetcolor);
clock->labelfont(FL_BOLD);
clock->labelcolor(FL_YELLOW);
clock->selection_color(FL_RED);
clock->tooltip("When locked, value can't be reset from the analysis screen");
clock->callback(lock_cb,(void *)ENERC_KCAL);
alock = new Nut_LightButton(column[3], row[0], 2*w/19, h/22, "AUTO-SET");
alock->color(widgetcolor);
alock->labelfont(FL_BOLD);
alock->labelcolor(FL_YELLOW);
alock->selection_color(FL_RED);
alock->tooltip("Determine the optimal calorie level from my daily weight log entries that include body fat percentage and clear the weight log upon success to start new cycles of logging and calorie adjustments.  I will eat according to the calorie level shown.");
alock->callback(alock_cb,(void *)clock);                                 

 {
 Nut_Box *o = new Nut_Box(column[0], row[1], w/6, h/22, "Omega-6/3 Balance");
 o->labelcolor(FL_YELLOW);
 o->labelfont(FL_BOLD);
 o->align(FL_ALIGN_INSIDE|FL_ALIGN_LEFT);
 }
ovo = new DVValueOutput(column[1], row[1], w/10, h/22, widgetcolor, FAPU);
 {
 Nut_Box *o = new Nut_Box(column[2], row[1], w/3, h/22, "takes effect after analysis resets");
 o->align(FL_ALIGN_INSIDE|FL_ALIGN_LEFT);
 o->labelfont(FL_BOLD);
 o->labelcolor(FL_YELLOW);
 }

 {
 Nut_Box *o = new Nut_Box(column[0], row[2], w/6, h/22, "LA:ALA Ratio");
 o->labelcolor(FL_YELLOW);
 o->labelfont(FL_BOLD);
 o->align(FL_ALIGN_INSIDE|FL_ALIGN_LEFT);
 }
lvo = new DVValueOutput(column[1], row[2], w/10, h/22, widgetcolor, ALA);
 {
 Nut_Box *o = new Nut_Box(column[2], row[2], w/3, h/22, "takes effect after analysis resets");
 o->align(FL_ALIGN_INSIDE|FL_ALIGN_LEFT);
 o->labelfont(FL_BOLD);
 o->labelcolor(FL_YELLOW);
 }

 {
 Nut_Box *o = new Nut_Box(column[0], row[3], w/6, h/22, "Maximum EPA+DHA");
 o->labelcolor(FL_YELLOW);
 o->labelfont(FL_BOLD);
 o->align(FL_ALIGN_INSIDE|FL_ALIGN_LEFT);
 }
mvo = new DVValueOutput(column[1], row[3], w/10, h/22, widgetcolor, EPA);
 {
 Nut_Box *o = new Nut_Box(column[2], row[3], w/3, h/22, "takes effect after analysis resets");
 o->align(FL_ALIGN_INSIDE|FL_ALIGN_LEFT);
 o->labelfont(FL_BOLD);
 o->labelcolor(FL_YELLOW);
 }

 {
 Nut_Box *o = new Nut_Box(column[0], row[4], w/6, h/22, "Saturated Fat %");
 o->labelcolor(FL_YELLOW);
 o->labelfont(FL_BOLD);
 o->align(FL_ALIGN_INSIDE|FL_ALIGN_LEFT);
 }
svo = new DVValueOutput(column[1], row[4], w/10, h/22, widgetcolor, FASAT);
slock = new Nut_LightButton(column[2], row[4], w/15, h/22, "LOCK");
slock->color(widgetcolor);
slock->labelfont(FL_BOLD);
slock->labelcolor(FL_YELLOW);
slock->selection_color(FL_RED);
slock->tooltip("When locked, value can't be reset from the analysis screen");
slock->callback(lock_cb,(void *)FASAT);

 {
 Nut_Box *o = new Nut_Box(column[0], row[5], w/6, h/22, "Fiber");
 o->labelcolor(FL_YELLOW);
 o->labelfont(FL_BOLD);
 o->align(FL_ALIGN_INSIDE|FL_ALIGN_LEFT);
 }
fvo = new DVValueOutput(column[1], row[5], w/10, h/22, widgetcolor, FIBTG);
flock = new Nut_LightButton(column[2], row[5], w/15, h/22, "LOCK");
flock->color(widgetcolor);
flock->labelfont(FL_BOLD);
flock->labelcolor(FL_YELLOW);
flock->selection_color(FL_RED);
flock->tooltip("When locked, value can't be reset from the analysis screen");
flock->callback(lock_cb,(void *)FIBTG);

 {
 Nut_Box *o = new Nut_Box(column[0], row[6], w/6, h/22, "Non-Fiber Carb");
 o->labelcolor(FL_YELLOW);
 o->labelfont(FL_BOLD);
 o->align(FL_ALIGN_INSIDE|FL_ALIGN_LEFT);
 }
nvo = new DVValueOutput(column[1], row[6], w/10, h/22, widgetcolor, CHO_NONFIB);
nlock = new Nut_LightButton(column[2], row[6], w/15, h/22, "LOCK");
nlock->color(widgetcolor);
nlock->labelfont(FL_BOLD);
nlock->labelcolor(FL_YELLOW);
nlock->selection_color(FL_RED);
nlock->tooltip("When locked, value can't be reset from the analysis screen");
nlock->callback(lock_cb,(void *)CHOCDF);

 {
 Nut_Box *o = new Nut_Box(column[0], row[7], w/6, h/22, "Protein");
 o->labelcolor(FL_YELLOW);
 o->labelfont(FL_BOLD);
 o->align(FL_ALIGN_INSIDE|FL_ALIGN_LEFT);
 }
pvo = new DVValueOutput(column[1], row[7], w/10, h/22, widgetcolor, PROCNT);
plock = new Nut_LightButton(column[2], row[7], w/15, h/22, "LOCK");
plock->color(widgetcolor);
plock->labelfont(FL_BOLD);
plock->labelcolor(FL_YELLOW);
plock->selection_color(FL_RED);
plock->tooltip("When locked, value can't be reset from the analysis screen");
plock->callback(lock_cb,(void *)PROCNT);

 {
 Nut_Button *o = new Nut_Button(column[1]-w/50, row[8]+h/33, w/6+w/48, h/22, "Restore All Defaults");
 o->labelfont(FL_BOLD);
 o->callback(restore_global_defaults_cb,(void *)this);
 }

settings->end(); 

weightlog = new Fl_Group(x, y, w, h);
weightlog->color(widgetcolor);
 {
 Nut_Button *o = new Nut_Button(x+96*w/128, y+h/640+h/22, 9*w/40, h/22, "Program Settings");
 o->color(fl_lighter(widgetcolor));
 o->labelfont(FL_BOLD);
 o->labelcolor(FL_YELLOW);
 o->clear_visible_focus();
 o->box(FL_ROUND_UP_BOX);
 o->callback(settings_cb,this);
 }
 weightlogwidget = new WeightLogWidget(x+w/24, y+h/640+3*h/22, w/4, 75*h/88, widgetcolor, this);
 weightcharts = new WeightCharts(x+2*w/24+w/4, y+h/640+3*h/22, 2*w/3, 75*h/88, widgetcolor);
weightlog->end(); 

this->end();
value(settings);
update();
}

void PersonalOptions::update(void)
{
cvo->update();
ovo->update();
lvo->update();
mvo->update();
svo->update();
fvo->update();
nvo->update();
pvo->update();
if (options.locknuts[ENERC_KCAL] && options.autocal != 2) clock->value(1);
else clock->value(0);
if (options.autocal == 2) alock->value(1);
else alock->value(0);
if (options.locknuts[FASAT]) slock->value(1);
else slock->value(0);
if (options.locknuts[FIBTG]) flock->value(1);
else flock->value(0);
if (options.locknuts[CHOCDF]) nlock->value(1);
else nlock->value(0);
if (options.locknuts[PROCNT]) plock->value(1);
else plock->value(0);
}

void PersonalOptions::initializeWeightLog(void)
{
read_WLOG(&n,&nfat,&weightslope,&fatslope,&weightyintercept,&fatyintercept,&earliestrdate);
weightlogwidget->initialize(&n,&nfat,&weightyintercept,&fatyintercept);
weightcharts->populate(&n,&nfat,&weightslope,&fatslope,&weightyintercept,&fatyintercept,&earliestrdate);
redraw_whole_schmeer();
next();
}

void PersonalOptions::initializeWeightLogAutoCal(void)
{
read_WLOG(&n,&nfat,&weightslope,&fatslope,&weightyintercept,&fatyintercept,&earliestrdate);
float leanslope = weightslope - fatslope;
if (nfat > 1)
 {
 if (fatslope > 0)
  {
  options.abnuts[ENERC_KCAL] -= 20;
  options.wltweak = 1;
  }
 else if (fatslope < 0 && leanslope < 0)
  {
  options.abnuts[ENERC_KCAL] += 20;
  options.wltweak = 1;
  }
 else if (fatslope < 0 && leanslope > 0 && options.wltweak)
  {
  options.wltweak = 0;
  float weight = 0, bf;
  char meal_date[9];
  if (options.autocal == 2)
   {
   options.lastwlw = weightyintercept;
   options.lastwlbfp = 100 * fatyintercept / weightyintercept;
   }
  if (n > 0) write_WLOG(&weight,&bf,meal_date);
  read_WLOG(&n,&nfat,&weightslope,&fatslope,&weightyintercept,&fatyintercept,&earliestrdate);
  }
 auto_cal(&options.abnuts[ENERC_KCAL],0,&food_work);
 write_OPTIONS();
 this->update();
 dv_change_minus_po();
 }

weightlogwidget->initialize(&n,&nfat,&weightyintercept,&fatyintercept);
weightcharts->populate(&n,&nfat,&weightslope,&fatslope,&weightyintercept,&fatyintercept,&earliestrdate);
redraw_whole_schmeer();
next();
}

void PersonalOptions::clearWeightLog(void)
{
float weight = 0, bf;
char meal_date[9];
if (n > 0) write_WLOG(&weight,&bf,meal_date);
initializeWeightLog();
}

void PersonalOptions::accept(double w, double bodyfat)
{
float weight = (float) w, bf = (float) bodyfat;
char meal_date[9];
if (weight == 0) return;
today(meal_date);
write_WLOG(&weight,&bf,meal_date);
if (options.autocal != 2) initializeWeightLog(); 
else initializeWeightLogAutoCal();
}
